*------------------------------------------------------------------------*
*    This file is driver program for Decays with 3Body integrations      *
*    in the final state.                                                 *
*                                                                        *
*    It uses Vegas or Gauss integration. For the Vegas integration makes *
*    use of the Cuba 4.2 library                                         *
*    For the Gauss integration a subroutine with the same calling        *
*    structure is provided.                                              *
*                                                                        *
*    All the methods use integration in an hypercube                     *
*    of unit length. So the program as to map the intervals into [0,1].  *
*    The limits of the intervals are stored in the arrays xmin and xmax. *
*                                                                        *
*    This package consists of the following files:                       *
*                                                                        *
*    1) Driver3BodyDecays.f (this file)                                  *
*    2) Integrations3Body.f: The integration routines. In principle      *
*                    the user does not have to change here.              *
*    3) MsqAvg.f: User provided averaged amplitude squared. It must have *
*              the following calling structure:                          *
*                                                                        *
*      call  Msq(M0,m1,m2,m3,P0q1,P0q2,P0q3,q1q2,q1q3,q2q3,              *
*     &     m12,P0P0,q1q1,q2q2,q3q3,EPSP0q1q2q3)                         *
*      real*8 rs,m1,m2,m3                                                *
*      real*8 P0q1,P0q2,P0q3,q1q2,q1q3,q2q3,                             *
*     &     m12,P0P0,q1q1,q2q2,q3q3,EPSP0q1q2q3                          *
*      ...                                                               *
*      common/parameters/xrs,xma,xmb,xm1,xm2,xm3                         *
*                                                                        *
*   where  P0q1,P0q2,.. are all the Lorentz invariant scalar products    *
*   An example is provided for the case eE-> ZZH. Finally a Makefile     *      
*   is also provided showing how to call the Cuba library                *      
*                                                                        *
*   Author: Jorge C. Romão                                               *
*   Date: 04.02.2022                                                     *
*   Email: jorge.romao@tecnico.ulisboa.pt                                *
*------------------------------------------------------------------------*
      program Decays3Body
      implicit none
      integer i,method,npoints

      real*8 M0,xrs,xma,xmb,xm1,xm2,xm3
      real*8 rs,ma,mb,m1,m2,m3
      real*8 thetaw,g2,gp,mw,mz,gw,gz,sw2,ee,GF,pi
      real*8 width
      character*7 DecayOrXS
      
      common/data/mw,mz,gw,gz,sw2,ee,GF,pi
      common/parameters/xrs,xma,xmb,xm1,xm2,xm3
      common/Type/DecayOrXS
      
      open(10,file='res.dat',status='unknown')

*      
***   Choose DecayOrXs='XSect23' or DecayOrXs='Decay13' 
*
      DecayOrXs='Decay13'
      
*------------------------------------------------------------------------*
***     SM Parameters
*------------------------------------------------------------------------*
      sw2=0.23d0
      thetaw=asin(sqrt(sw2))
      mz= 91.187D0
      mw=mz*cos(thetaw)
      pi=acos(-1.d0)
      GF=1.16639d-05
      gw=2.06d0
      gz=2.49d0
      g2=sqrt(sqrt(32.d0)*mw**2*GF)
      gp=g2*tan(thetaw)
      ee=g2*sin(thetaw)
*------------------------------------------------------------------------*
***  SM Inputs to compare with CalcHEP
*------------------------------------------------------------------------*
      if(.True.)then
         sw2=0.474d0**2
         GF=1.1951d-05
         MZ=91.29264d0
         ee=0.3133d0
         MW=80.385d0
      endif
*------------------------------------------------------------------------*
*     Parameters for the process:
*
*     P0(M0) -> q1(m1) + q2(m2) + q3(m3)
*
*------------------------------------------------------------------------*
      xm1=0d0
      xm2=0d0
      xm3=MW
*------------------------------------------------------------------------*
*     Parameters for integration:
*
*     method=1 -> gauss method, should give
*                npoints=4,6,8,10,16,20,24,32,64,128. Defaults to 16
*     method=2 -> vegas method
*------------------------------------------------------------------------*
      method=2
      if(method.eq.1)then
         npoints=32
      else
         npoints=16
      endif

      
      M0=98d0      
      do i=1,51
         M0=M0+2.d0
         xrs=M0
*------------------------------------------------------------------------* 
*
***   Call xsection subroutine   
*
*------------------------------------------------------------------------*
         call DecayWidth(width,method,npoints)

***   Warning: identical particles -> factor 1/2
***   Result in GeV

         write(10,99)M0,width

      enddo

 99   format(3(e14.5,2x))
      end
